/************************************************************************/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/


#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include "llog.h"
#include "libsl.h"
#include "sd_table.h"


/* Function prototypes */
static int validate_args(int sock, char *dest, int size);
/*static int remove_line(char *dest, int size);*/



int sl_SendBuff(int sock, char Buffer[], int len)
{
    if((!Buffer) || (strlen(Buffer) == 0))
    {
        /* Error */
        return -1;
    }
    
    if (send(sock, Buffer, len, 0) == -1) 
    {
        /* the send failed. */
        return -1;
    }
    return 0;
}


int sl_GetBuff(int sock, char *buff, int size)
{
    int     msg_len;
    char    dbgstr[100]; /* used for debugging */
    
    /* Make sure arguments are reasonable. */
    if( -1 == validate_args(sock, buff, size)) return -1;

    msg_len = recv(sock, buff, size, 0);

    if (msg_len == -1)
    {
        /* We got a error */
        Llog_warning("recv on socket failed\n");
        return(-1);
    }
    
    /* So we got some data... */
    memset(dbgstr, 0, sizeof(dbgstr));
    memcpy(dbgstr, buff, msg_len);
    Llog_debug("  recv returns %d bytes <%s>\n", msg_len, dbgstr);
    return msg_len;
}



int sl_SendLine(int sock, char *line)
{
    int len;
    int res;
    
    if((!line) || (strlen(line) == 0))
    {
        /* Error */
        return -1;
    }
    
    len = strlen(line);
    res = sl_SendBuff(sock, line, len);   
    
    return res;
}    



 
/*  get a line of text.  Keep reading until we get the \n terminator */
/* "size" should include the terminating null on a string */
int sl_GetLine(int sock, char *dest, int size)
{
    int     bytes_recvd;
    int     remaining_bytes;
    int     res;
    sdent  *sd_ptr = NULL;
    
    if( -1 == validate_args(sock, dest, size)) return -1;
    
    sd_ptr = sd_get_sdent(sock);
    if( !sd_ptr)
    {
        /*error*/
        Llog_warning("sd_get_sdent failed\n");
        return -1;
    }
    
    
    /* If we can remove a full line, remove and return it */
    res = sd_remove_line(sd_ptr, dest, size);
    if( res > 0 )
    {
        return res;
    }
    
    while( !index(sd_ptr->buff, '\n') ) 
    {   
        /* Read until we get a new line */
        remaining_bytes = BUFFER_SIZE - (sd_ptr->buff_ptr - sd_ptr->buff);
        if( remaining_bytes <= 1 )
        {
            /* no space left, no point in reading more*/
            Llog_warning("buffer overflow\n");
            break;    
        }

        bytes_recvd = sl_GetBuff(sock, sd_ptr->buff_ptr, remaining_bytes);

        if(bytes_recvd < 0)
        {
            /* read failed return ERROR */
            Llog_debug("  sl_GetBuff returns < 0 bytes\n");
            break;
        }
        else if (0 == bytes_recvd)
        {
            /* end of connection, terminated by client.)  */
            break;
        }
        sd_ptr->buff_ptr += bytes_recvd;
    }
    
    
    /* If sd_remove_line() can not find a full line here then if    */
    /* must be that the client disconnected.  The while loop above  */
    /* will only terminate if a full line is found or on a          */
    /* disconect (EOF)                                              */
    Llog_debug(" returning line\n");
    return sd_remove_line(sd_ptr, dest, size);
}


static int validate_args(int sock, char *dest, int size)
{   
    if(sock <= 0)
    {
        /* Invalid SD */
        Llog_warning("sl_GetLine called with invalid socket %d\n", sock);
        return -1;
    }
    
    if(!dest)
    {
        /* Error */
        Llog_warning("sl_GetLine called with null dest\n");
        return -1;
    }
    
    if(size < 1)
    {
        /* Unreasonable size */
        Llog_warning("sl_GetLine called Unreasonable size\n");
        return -1;
    }
    
    return 0;
}
