
	/************************************************************************
	 * A simple little client program for the "tait" Mark IV control program.
	 * It sends its command-line arguments as a single line to a waiting
	 * Mark IV server.
	 *
	 * Usage:           taitcl   arg ...
	 *
	 * The program opens a socket, sends its line, waits for the return,
	 * prints it to stdout, then quits.
	 * It doesn't stay open to send another line.
	 *
	 * Based heavily on code from Chris Albertson.
	 *
	 * MWR 12/16/2001
	 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "libsl.h"
#include "llog.h"
#include "tait.h"

#define MSGLEN 500

int main(int argc, char *argv[])
{
	 char message[MSGLEN];
	 int i, pos, len;
	 int port = DEFAULT_PORT;
    int sd;

   if (argc < 2) {
      fprintf(stderr, "usage: taitcl [-p port] message for server\n");
      exit(1);
   }

	/* if given -p flag, set the port number */
	if (strcmp(argv[1], "-p") == 0) {
		if (sscanf(argv[2], "%d", &port) != 1) {
			fprintf(stderr, "taitcl: invalid port number in arg %s\n", 
					argv[2]);
			exit(1);
		}
	}

	/* figure out how much space we need for the message */
	len = 0;
	for (i = 1; i < argc; i++) {
		if (strcmp(argv[i], "-p") == 0) {
			i += 1;
			continue;
		}
		len += strlen(argv[i]) + 1;
	}
	if (len > MSGLEN - 2) {
		fprintf(stderr, "taitcl: command line too long; only %d chars allowed\n",
					MSGLEN);
		exit(1);
	}
	
	/* build the message */
	message[0] = '\0';
	for (i = 1; i < argc; i++) {
		if (strcmp(argv[i], "-p") == 0) {
			i += 1;
			continue;
		}
		strcat(message, argv[i]);
		strcat(message, " ");
	}
	pos = strlen(message);
	shAssert(pos < MSGLEN - 2);
	message[pos] = '\n';
	message[pos + 1] = '\0';

	Llog_setlvl(LL_DEBUG);
    
	Llog_debug("Calling sl_Setup_Client_Socket...\n");
	sd = sl_Setup_Client_Socket(NULL, port, NULL); 
	Llog_debug("    sl_Setup_Server_Socket returns %d\n", sd);
    
	sl_SendLine(sd, message);
	exit(0);
}



